@@ -6,7 +6,6 @@
 #include "mtx/identifiers.hpp"
 
 using json = nlohmann::json;
-
 namespace mtx {
 namespace events {
 
@@ -24,6 +23,10 @@ enum class EventType
         KeyVerificationKey,
         /// m.key.verification.mac
         KeyVerificationMac,
+        /// m.reaction,
+        Reaction,
+        /// m.room_key
+        RoomKey,
         /// m.room_key_request
         RoomKeyRequest,
         /// m.room.aliases
@@ -64,6 +67,8 @@ enum class EventType
         Sticker,
         // m.tag
         Tag,
+        // m.presence
+        Presence,
         // m.push_rules
         PushRules,
         // Unsupported event
@@ -90,6 +95,8 @@ struct Event
         //! This *should* be namespaced similar to Java package
         //! naming conventions e.g. 'com.example.subdomain.event.type'
         EventType type;
+        //! Contains the fully-qualified ID of the user who sent this event.
+        std::string sender;
 };
 
 template<class Content>
@@ -97,93 +104,8 @@ void
 to_json(json &obj, const Event<Content> &event)
 {
         obj["content"] = event.content;
-
-        switch (event.type) {
-        case EventType::KeyVerificationStart:
-                obj["type"] = "m.key.verification.start";
-                break;
-        case EventType::KeyVerificationAccept:
-                obj["type"] = "m.key.verification.accept";
-                break;
-        case EventType::KeyVerificationMac:
-                obj["type"] = "m.key.verification.mac";
-                break;
-        case EventType::KeyVerificationKey:
-                obj["type"] = "m.key.verification.key";
-                break;
-        case EventType::KeyVerificationCancel:
-                obj["type"] = "m.key.verification.cancel";
-                break;
-        case EventType::KeyVerificationRequest:
-                obj["type"] = "m.key.verification.request";
-                break;
-        case EventType::RoomKeyRequest:
-                obj["type"] = "m.room_key_request";
-                break;
-        case EventType::RoomAliases:
-                obj["type"] = "m.room.aliases";
-                break;
-        case EventType::RoomAvatar:
-                obj["type"] = "m.room.avatar";
-                break;
-        case EventType::RoomCanonicalAlias:
-                obj["type"] = "m.room.canonical_alias";
-                break;
-        case EventType::RoomCreate:
-                obj["type"] = "m.room.create";
-                break;
-        case EventType::RoomEncrypted:
-                obj["type"] = "m.room.encrypted";
-                break;
-        case EventType::RoomEncryption:
-                obj["type"] = "m.room.encryption";
-                break;
-        case EventType::RoomGuestAccess:
-                obj["type"] = "m.room.guest_access";
-                break;
-        case EventType::RoomHistoryVisibility:
-                obj["type"] = "m.room.history_visibility";
-                break;
-        case EventType::RoomJoinRules:
-                obj["type"] = "m.room.join_rules";
-                break;
-        case EventType::RoomMember:
-                obj["type"] = "m.room.member";
-                break;
-        case EventType::RoomMessage:
-                obj["type"] = "m.room.message";
-                break;
-        case EventType::RoomName:
-                obj["type"] = "m.room.name";
-                break;
-        case EventType::RoomPowerLevels:
-                obj["type"] = "m.room.power_levels";
-                break;
-        case EventType::RoomTopic:
-                obj["type"] = "m.room.topic";
-                break;
-        case EventType::RoomRedaction:
-                obj["type"] = "m.room.redaction";
-                break;
-        case EventType::RoomPinnedEvents:
-                obj["type"] = "m.room.pinned_events";
-                break;
-        case EventType::RoomTombstone:
-                obj["type"] = "m.room.tombstone";
-                break;
-        case EventType::Sticker:
-                obj["type"] = "m.sticker";
-                break;
-        case EventType::Tag:
-                obj["type"] = "m.tag";
-                break;
-        case EventType::PushRules:
-                obj["type"] = "m.push_rules";
-                break;
-        case EventType::Unsupported:
-        default:
-                break;
-        }
+        obj["sender"]  = event.sender;
+        obj["type"]    = ::mtx::events::to_string(event.type);
 }
 
 template<class Content>
@@ -192,6 +114,35 @@ from_json(const json &obj, Event<Content> &event)
 {
         event.content = obj.at("content").get<Content>();
         event.type    = getEventType(obj.at("type").get<std::string>());
+        event.sender  = obj.value("sender", "");
+}
+
+//! Extension of the Event type for device events.
+template<class Content>
+struct DeviceEvent : public Event<Content>
+{
+        std::string sender;
+};
+
+template<class Content>
+void
+from_json(const json &obj, DeviceEvent<Content> &event)
+{
+        Event<Content> base_event = event;
+        from_json(obj, base_event);
+        event.content = base_event.content;
+        event.type    = base_event.type;
+        event.sender  = obj.at("sender");
+}
+
+template<class Content>
+void
+to_json(json &obj, const DeviceEvent<Content> &event)
+{
+        Event<Content> base_event = event;
+        to_json(obj, base_event);
+
+        obj["sender"] = event.sender;
 }
 
 struct UnsignedData
@@ -249,7 +200,6 @@ to_json(json &obj, const UnsignedData &event)
 template<class Content>
 struct StrippedEvent : public Event<Content>
 {
-        std::string sender;
         std::string state_key;
 };
 
@@ -257,9 +207,9 @@ template<class Content>
 void
 from_json(const json &obj, StrippedEvent<Content> &event)
 {
-        event.content   = obj.at("content").get<Content>();
-        event.type      = getEventType(obj.at("type").get<std::string>());
-        event.sender    = obj.at("sender");
+        Event<Content> &base = event;
+        from_json(obj, base);
+
         event.state_key = obj.at("state_key");
 }
 
@@ -270,7 +220,6 @@ to_json(json &obj, const StrippedEvent<Content> &event)
         Event<Content> base_event = event;
         to_json(obj, base_event);
 
-        obj["sender"]    = event.sender;
         obj["state_key"] = event.state_key;
 }
 
@@ -282,8 +231,6 @@ struct RoomEvent : public Event<Content>
         std::string event_id;
         //! The ID of the room associated with this event.
         std::string room_id;
-        //! Contains the fully-qualified ID of the user who sent this event.
-        std::string sender;
         //! Timestamp in milliseconds on originating homeserver
         //! when this event was sent.
         uint64_t origin_server_ts;
@@ -296,11 +243,10 @@ template<class Content>
 void
 from_json(const json &obj, RoomEvent<Content> &event)
 {
-        event.content = obj.at("content").get<Content>();
-        event.type    = getEventType(obj.at("type").get<std::string>());
+        Event<Content> &base = event;
+        from_json(obj, base);
 
         event.event_id         = obj.at("event_id");
-        event.sender           = obj.at("sender");
         event.origin_server_ts = obj.at("origin_server_ts");
 
         // SPEC_BUG: Not present in the state array returned by /sync.
@@ -322,7 +268,6 @@ to_json(json &obj, const RoomEvent<Content> &event)
                 obj["room_id"] = event.room_id;
 
         obj["event_id"]         = event.event_id;
-        obj["sender"]           = event.sender;
         obj["unsigned"]         = event.unsigned_data;
         obj["origin_server_ts"] = event.origin_server_ts;
 }
@@ -350,17 +295,8 @@ template<class Content>
 void
 from_json(const json &obj, StateEvent<Content> &event)
 {
-        event.content          = obj.at("content").get<Content>();
-        event.event_id         = obj.at("event_id");
-        event.origin_server_ts = obj.at("origin_server_ts");
-        event.sender           = obj.at("sender");
-        event.type             = getEventType(obj.at("type").get<std::string>());
-
-        if (obj.find("room_id") != obj.end())
-                event.room_id = obj.at("room_id");
-
-        if (obj.find("unsigned") != obj.end())
-                event.unsigned_data = obj.at("unsigned");
+        RoomEvent<Content> &base = event;
+        from_json(obj, base);
 
         event.state_key = obj.at("state_key").get<std::string>();
 }
@@ -387,17 +323,8 @@ template<class Content>
 void
 from_json(const json &obj, RedactionEvent<Content> &event)
 {
-        event.content          = obj.at("content").get<Content>();
-        event.event_id         = obj.at("event_id");
-        event.origin_server_ts = obj.at("origin_server_ts");
-        event.sender           = obj.at("sender");
-        event.type             = getEventType(obj.at("type").get<std::string>());
-
-        if (obj.find("unsigned") != obj.end())
-                event.unsigned_data = obj.at("unsigned");
-
-        if (obj.find("room_id") != obj.end())
-                event.room_id = obj.at("room_id");
+        RoomEvent<Content> &base = event;
+        from_json(obj, base);
 
         event.redacts = obj.at("redacts").get<std::string>();
 }
@@ -419,17 +346,8 @@ template<class Content>
 void
 from_json(const json &obj, EncryptedEvent<Content> &event)
 {
-        event.content          = obj.at("content").get<Content>();
-        event.event_id         = obj.at("event_id");
-        event.origin_server_ts = obj.at("origin_server_ts");
-        event.sender           = obj.at("sender");
-        event.type             = getEventType(obj.at("type").get<std::string>());
-
-        if (obj.find("unsigned") != obj.end())
-                event.unsigned_data = obj.at("unsigned");
-
-        if (obj.find("room_id") != obj.end())
-                event.room_id = obj.at("room_id");
+        RoomEvent<Content> &base = event;
+        from_json(obj, base);
 }
 
 enum class MessageType
