@@ -1,9 +1,12 @@
 #pragma once
 
+#if __has_include(<nlohmann/json_fwd.hpp>)
+#include <nlohmann/json_fwd.hpp>
+#else
 #include <nlohmann/json.hpp>
-#include <string>
+#endif
 
-using json = nlohmann::json;
+#include <string>
 
 namespace mtx {
 namespace events {
@@ -42,14 +45,17 @@ struct Member
         //! with the intention of being a direct chat.
         bool is_direct = false;
 
+        //! reason for the membership change, empty in most cases
+        std::string reason;
+
         /* ThirdPartyInvite third_party_invite; */
 };
 
 void
-from_json(const json &obj, Member &member);
+from_json(const nlohmann::json &obj, Member &member);
 
 void
-to_json(json &obj, const Member &member);
+to_json(nlohmann::json &obj, const Member &member);
 
 } // namespace state
 } // namespace events
