@@ -10,7 +10,9 @@
 #include "mtx/events.hpp"             // for EventType, to_string, json
 #include "mtx/events/collections.hpp" // for TimelineEvents
 #include "mtx/identifiers.hpp"        // for User
+#include "mtx/identifiers.hpp"        // for Class user
 #include "mtx/pushrules.hpp"
+#include "mtx/requests.hpp"
 #include "mtx/responses/empty.hpp"   // for Empty, Logout, RoomInvite
 #include "mtxclient/http/errors.hpp" // for ClientError
 #include "mtxclient/utils.hpp"       // for random_token, url_encode, des...
@@ -51,6 +53,7 @@ struct GroupProfile;
 struct JoinedGroups;
 struct KeyChanges;
 struct Login;
+struct LoginFlows;
 struct Messages;
 struct Notifications;
 struct Profile;
@@ -106,6 +109,8 @@ struct SyncOpts
         uint16_t timeout = 30'000;
         //! Wheter to include the full state of each room.
         bool full_state = false;
+        //! Explicitly set the presence of the user
+        std::optional<mtx::presence::PresenceState> set_presence;
 };
 
 //! Configuration for the /messages endpoint.
@@ -193,6 +198,12 @@ public:
                    const std::string &device_name,
                    Callback<mtx::responses::Login> cb);
         void login(const mtx::requests::Login &req, Callback<mtx::responses::Login> cb);
+
+        //! Get the supported login flows
+        void get_login(Callback<mtx::responses::LoginFlows> cb);
+        //! Get url to navigate to for sso login flow
+        //! Open this in a browser
+        std::string login_sso_redirect(std::string redirectUrl);
         //! Lookup real server to connect to.
         //! Call set_server with the returned homeserver url after this
         void well_known(Callback<mtx::responses::WellKnown> cb);
@@ -279,6 +290,17 @@ public:
         void get_profile(const std::string &user_id, Callback<mtx::responses::Profile> cb);
         //! Get user avatar URL.
         void get_avatar_url(const std::string &user_id, Callback<mtx::responses::AvatarUrl> cb);
+
+        //! List the tags set by a user on a room.
+        void get_tags(const std::string &room_id, Callback<mtx::events::account_data::Tags> cb);
+        //! Add a tag to the room.
+        void put_tag(const std::string &room_id,
+                     const std::string &tag_name,
+                     const mtx::events::account_data::Tag &tag,
+                     ErrCallback cb);
+        //! Remove a tag from the room.
+        void delete_tag(const std::string &room_id, const std::string &tag_name, ErrCallback cb);
+
         //! Create a room with the given options.
         void create_room(const mtx::requests::CreateRoom &room_options,
                          Callback<mtx::responses::CreateRoom> cb);
@@ -356,6 +378,15 @@ public:
         void start_typing(const std::string &room_id, uint64_t timeout, ErrCallback cb);
         //! Remove typing notifications from the room.
         void stop_typing(const std::string &room_id, ErrCallback cb);
+
+        //! Get presence of a user
+        void presence_status(const std::string &user_id,
+                             Callback<mtx::events::presence::Presence> cb);
+        //! Set presence of the user
+        void put_presence_status(mtx::presence::PresenceState state,
+                                 std::optional<std::string> status_msg,
+                                 ErrCallback cb);
+
         //! Get a single event.
         void get_event(const std::string &room_id,
                        const std::string &event_id,
@@ -396,6 +427,20 @@ public:
         {
                 send_to_device(event_type, generate_txn_id(), body, cb);
         }
+        //! Send send-to-device events to a set of client devices with a specified transaction id.
+        template<typename EventContent, mtx::events::EventType Event>
+        void send_to_device(
+          const std::string &txid,
+          const std::map<mtx::identifiers::User, std::map<std::string, EventContent>> &messages,
+          ErrCallback callback)
+        {
+                json j;
+                for (const auto &[user, deviceToMessage] : messages)
+                        for (const auto &[deviceid, message] : deviceToMessage)
+                                j["messages"][user.to_string()][deviceid] = message;
+
+                send_to_device(mtx::events::to_string(Event), txid, j, callback);
+        }
 
         //
         // Group related endpoints.
@@ -653,8 +698,9 @@ mtx::http::Client::send_state_event(const std::string &room_id,
                                     const Payload &payload,
                                     Callback<mtx::responses::EventId> callback)
 {
-        const auto api_path = "/client/r0/rooms/" + room_id + "/state/" +
-                              mtx::events::to_string(Event) + "/" + state_key;
+        const auto api_path = "/client/r0/rooms/" + mtx::client::utils::url_encode(room_id) +
+                              "/state/" + mtx::events::to_string(Event) + "/" +
+                              mtx::client::utils::url_encode(state_key);
 
         put<Payload, mtx::responses::EventId>(api_path, payload, callback);
 }
