@@ -100,7 +100,7 @@ from_json(const json &obj, Encrypted &content)
         content.session_id = obj.at("session_id").get<std::string>();
 
         if (obj.count("m.relates_to") != 0)
-                content.relates_to = obj.at("m.relates_to").get<common::RelatesTo>();
+                content.relates_to = obj.at("m.relates_to").get<common::ReplyRelatesTo>();
 }
 
 void
@@ -137,18 +137,16 @@ to_json(json &obj, const RoomKey &event)
 void
 from_json(const json &obj, KeyRequest &event)
 {
-        event.sender               = obj.at("sender");
-        event.type                 = mtx::events::getEventType(obj.at("type").get<std::string>());
-        event.request_id           = obj.at("content").at("request_id");
-        event.requesting_device_id = obj.at("content").at("requesting_device_id");
+        event.request_id           = obj.at("request_id");
+        event.requesting_device_id = obj.at("requesting_device_id");
 
-        auto action = obj.at("content").at("action").get<std::string>();
+        auto action = obj.at("action").get<std::string>();
         if (action == "request") {
                 event.action     = RequestAction::Request;
-                event.room_id    = obj.at("content").at("body").at("room_id");
-                event.sender_key = obj.at("content").at("body").at("sender_key");
-                event.session_id = obj.at("content").at("body").at("session_id");
-                event.algorithm  = obj.at("content").at("body").at("algorithm");
+                event.room_id    = obj.at("body").at("room_id");
+                event.sender_key = obj.at("body").at("sender_key");
+                event.session_id = obj.at("body").at("session_id");
+                event.algorithm  = obj.at("body").at("algorithm");
         } else if (action == "request_cancellation") {
                 event.action = RequestAction::Cancellation;
         }
@@ -159,27 +157,25 @@ to_json(json &obj, const KeyRequest &event)
 {
         obj = json::object();
 
-        obj["sender"]  = event.sender;
-        obj["type"]    = to_string(event.type);
-        obj["content"] = json::object();
+        obj = json::object();
 
-        obj["content"]["request_id"]           = event.request_id;
-        obj["content"]["requesting_device_id"] = event.requesting_device_id;
+        obj["request_id"]           = event.request_id;
+        obj["requesting_device_id"] = event.requesting_device_id;
 
         switch (event.action) {
         case RequestAction::Request: {
-                obj["content"]["body"] = json::object();
+                obj["body"] = json::object();
 
-                obj["content"]["body"]["room_id"]    = event.room_id;
-                obj["content"]["body"]["sender_key"] = event.sender_key;
-                obj["content"]["body"]["session_id"] = event.session_id;
-                obj["content"]["body"]["algorithm"]  = "m.megolm.v1.aes-sha2";
+                obj["body"]["room_id"]    = event.room_id;
+                obj["body"]["sender_key"] = event.sender_key;
+                obj["body"]["session_id"] = event.session_id;
+                obj["body"]["algorithm"]  = "m.megolm.v1.aes-sha2";
 
-                obj["content"]["action"] = "request";
+                obj["action"] = "request";
                 break;
         }
         case RequestAction::Cancellation: {
-                obj["content"]["action"] = "request_cancellation";
+                obj["action"] = "request_cancellation";
                 break;
         }
         default:
