@@ -12,6 +12,7 @@
 #include "mtx/events/name.hpp"
 #include "mtx/events/pinned_events.hpp"
 #include "mtx/events/power_levels.hpp"
+#include "mtx/events/reaction.hpp"
 #include "mtx/events/redaction.hpp"
 #include "mtx/events/tag.hpp"
 #include "mtx/events/topic.hpp"
@@ -21,6 +22,7 @@
 using json = nlohmann::json;
 using namespace mtx::events::account_data;
 using namespace mtx::events::state;
+using namespace mtx::events::msg;
 
 namespace mtx {
 namespace responses {
@@ -148,6 +150,12 @@ struct TimelineEventVisitor
                 mtx::events::to_json(j, stickEv);
                 return j;
         };
+        json operator()(const events::RoomEvent<msgs::Reaction> &reactEv) const
+        {
+                json j;
+                mtx::events::to_json(j, reactEv);
+                return j;
+        };
         json operator()(const events::RoomEvent<msgs::Redacted> &redEv) const
         {
                 json j;
@@ -229,7 +237,7 @@ parse_room_account_data_events(
                 switch (type) {
                 case events::EventType::Tag: {
                         try {
-                                container.emplace_back(events::Event<Tag>(e));
+                                container.emplace_back(events::Event<Tags>(e));
                         } catch (json::exception &err) {
                                 log_error(err, e);
                         }
@@ -249,7 +257,8 @@ parse_room_account_data_events(
                 case events::EventType::KeyVerificationAccept:
                 case events::EventType::KeyVerificationKey:
                 case events::EventType::KeyVerificationMac:
-                case events::EventType::RoomKeyRequest:
+                case events::EventType::Presence:
+                case events::EventType::Reaction:
                 case events::EventType::RoomAliases:
                 case events::EventType::RoomAvatar:
                 case events::EventType::RoomCanonicalAlias:
@@ -259,14 +268,16 @@ parse_room_account_data_events(
                 case events::EventType::RoomGuestAccess:
                 case events::EventType::RoomHistoryVisibility:
                 case events::EventType::RoomJoinRules:
+                case events::EventType::RoomKey:
+                case events::EventType::RoomKeyRequest:
                 case events::EventType::RoomMember:
                 case events::EventType::RoomMessage:
                 case events::EventType::RoomName:
+                case events::EventType::RoomPinnedEvents:
                 case events::EventType::RoomPowerLevels:
+                case events::EventType::RoomRedaction:
                 case events::EventType::RoomTombstone:
                 case events::EventType::RoomTopic:
-                case events::EventType::RoomRedaction:
-                case events::EventType::RoomPinnedEvents:
                 case events::EventType::Sticker:
                 case events::EventType::Unsupported:
                         continue;
@@ -293,6 +304,15 @@ parse_timeline_events(const json &events,
                 const auto type = mtx::events::getEventType(e);
 
                 switch (type) {
+                case events::EventType::Reaction: {
+                        try {
+                                container.emplace_back(events::RoomEvent<events::msg::Reaction>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                }
                 case events::EventType::RoomAliases: {
                         try {
                                 container.emplace_back(events::StateEvent<Aliases>(e));
@@ -544,8 +564,10 @@ parse_timeline_events(const json &events,
                         break;
                 }
                 case events::EventType::RoomPinnedEvents:
+                case events::EventType::RoomKey:        // Not part of timeline or state
                 case events::EventType::RoomKeyRequest: // Not part of the timeline
                 case events::EventType::Tag:            // Not part of the timeline or state
+                case events::EventType::Presence:       // Not part of the timeline or state
                 case events::EventType::PushRules:      // Not part of the timeline or state
                 case events::EventType::Unsupported:
                 case events::EventType::KeyVerificationCancel:
@@ -559,6 +581,113 @@ parse_timeline_events(const json &events,
         }
 }
 
+void
+parse_device_events(const json &events,
+                    std::vector<mtx::events::collections::DeviceEvents> &container)
+{
+        container.clear();
+        container.reserve(events.size());
+        for (const auto &e : events) {
+                const auto type = mtx::events::getEventType(e);
+
+                switch (type) {
+                case events::EventType::RoomEncrypted: {
+                        try {
+                                const auto algo =
+                                  e.at("content").at("algorithm").get<std::string>();
+                                // Algorithm determines whether it's an olm or megolm event
+                                if (algo == "m.olm.v1.curve25519-aes-sha2") {
+                                        container.emplace_back(
+                                          events::DeviceEvent<OlmEncrypted>(e));
+                                } else if (algo == "m.megolm.v1.aes-sha2") {
+                                        container.emplace_back(events::DeviceEvent<Encrypted>(e));
+                                } else {
+                                        log_error("Invalid m.room.encrypted algorithm", e);
+                                        continue;
+                                }
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                }
+                case events::EventType::RoomKey: {
+                        try {
+                                container.emplace_back(events::DeviceEvent<RoomKey>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                }
+                case events::EventType::RoomKeyRequest: {
+                        try {
+                                container.emplace_back(events::DeviceEvent<KeyRequest>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                }
+                case events::EventType::KeyVerificationCancel: {
+                        try {
+                                container.emplace_back(
+                                  events::DeviceEvent<KeyVerificationCancel>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                }
+                case events::EventType::KeyVerificationRequest:
+                        try {
+                                container.emplace_back(
+                                  events::DeviceEvent<KeyVerificationRequest>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                case events::EventType::KeyVerificationStart:
+                        try {
+                                container.emplace_back(
+                                  events::DeviceEvent<KeyVerificationStart>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                case events::EventType::KeyVerificationAccept:
+                        try {
+                                container.emplace_back(
+                                  events::DeviceEvent<KeyVerificationAccept>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                case events::EventType::KeyVerificationKey:
+                        try {
+                                container.emplace_back(events::DeviceEvent<KeyVerificationKey>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                case events::EventType::KeyVerificationMac:
+                        try {
+                                container.emplace_back(events::DeviceEvent<KeyVerificationMac>(e));
+                        } catch (json::exception &err) {
+                                log_error(err, e);
+                        }
+
+                        break;
+                default:
+                        continue;
+                }
+        }
+}
+
 void
 parse_state_events(const json &events,
                    std::vector<mtx::events::collections::StateEvents> &container)
@@ -688,12 +817,15 @@ parse_state_events(const json &events,
                         break;
                 }
                 case events::EventType::Sticker:
+                case events::EventType::Reaction:
                 case events::EventType::RoomEncrypted:  /* Does this need to be here? */
+                case events::EventType::RoomKey:        // Not part of timeline or state
                 case events::EventType::RoomKeyRequest: // Not part of the timeline or state
                 case events::EventType::RoomMessage:
                 case events::EventType::RoomPinnedEvents:
                 case events::EventType::RoomRedaction:
                 case events::EventType::Tag:       // Not part of the timeline or state
+                case events::EventType::Presence:  // Not part of the timeline or state
                 case events::EventType::PushRules: // Not part of the timeline or state
                 case events::EventType::Unsupported:
                 case events::EventType::KeyVerificationCancel:
@@ -827,13 +959,16 @@ parse_stripped_events(const json &events,
                         break;
                 }
                 case events::EventType::Sticker:
+                case events::EventType::Reaction:
                 case events::EventType::RoomEncrypted:
                 case events::EventType::RoomEncryption:
                 case events::EventType::RoomMessage:
                 case events::EventType::RoomRedaction:
+                case events::EventType::RoomKey:        // Not part of timeline or state
                 case events::EventType::RoomKeyRequest: // Not part of the timeline or state
                 case events::EventType::RoomPinnedEvents:
                 case events::EventType::Tag:       // Not part of the timeline or state
+                case events::EventType::Presence:  // Not part of the timeline or state
                 case events::EventType::PushRules: // Not part of the timeline or state
                 case events::EventType::Unsupported:
                 case events::EventType::KeyVerificationCancel:
